/*---------------------------------------------------------------------------

   Product Name:      Daily Reading from Taiz
   Product Version:   1.1.0
   Coded By:          Matthew Atkins

   Revision History:

         25/09/2002 - Changed the startup registry key from HKEY_LOCAL_MACHINE to
                      HKEY_CURRENT_USER. NT didn't like it if you weren't logged on as the
                      Administrator.
                      Have also upgraded to Indy 9.0.1.
         13/01/2002 - Added PurifyReading function to remove control characters from readings.
         24/11/2001 - Press the 'space' or 'return' key while the
                      reading is focused now closes the program.
         01/09/2001 - Removed the display of percentage downloaded.
         31/08/2001 - Download panel now displays the total number of bytes to download
                      and the percentage already downloaded.
         27/08/2001 - Added a "Cancel" option to the popup menu.
                    - Set the "Copy" option to be the default on the popup menu.
         28/07/2001 - Added an Edit menu featuring a copy command to the menu bar.
                    - Added code to enable/disable the copy command.
         27/07/2001 - Added a TActionList to replace the default ^C command.
                    - Added a custom popup menu to redReading.
                    - Added code to enable/disable the copy command.
         24/07/2001 - Reworked the registry code in the frmMain::FormDestroy event.
                    - Changed date code to use Now().
                    - Added casts to TCursor in frmAbout (prevents compiler warnings).
         03/06/2001 - Slightly changed registry code.
         19/05/2001 - Discovered the FormStyle = fsStayOnTop property.
                    - Added menus.
                    - Added About Box.
                    - Added download screen/Indy HTTP download code.
                    - Split disply code into a separate function.
         13/05/2001 - Began coding...
                    - Readings stored in plain text file for portability.
                    - File I/O performed with fstream.
                    - TRegistry used for providing 'run@startup' functionality.

   (c) Vivace Software (UK) Ltd. 2001.

---------------------------------------------------------------------------*/
#include <vcl.h>
#pragma hdrstop

#include "main.h"
#include "about.h"
#include "download.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::cmdCloseClick(TObject *Sender)
{
   Application->Terminate();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
   Application->OnIdle = WhenIdle;
   Application->ShowHint = true;
   Application->HelpFile = ExtractFilePath(Application->ExeName) + "Taize.hlp";
   // Check for a 'run@startup' data value in the registry.
   TRegistry *theReg = new TRegistry();
   theReg->RootKey = HKEY_CURRENT_USER;
   theReg->OpenKey("Software\\Microsoft\\Windows\\CurrentVersion\\Run",false);
   if (theReg->ValueExists("Taize"))
      chkWindowsStartup->Checked = theReg->ReadString("Taize") == Application->ExeName;
   else
      chkWindowsStartup->Checked = false;
   delete theReg;
   // Ensure the window is in the centre of the desktop.
   Left = (Screen->Width / 2) - (Width / 2);
   Top = (Screen->Height / 2) - (Height /2);
   // Get todays date.
   lblDate->Caption = Now().FormatString("ddd d mmm yyyy");
   ReadingToGet = Now().FormatString("dd mm");
   ReadingFound = false;
   // Attempt to display the reading...
   ShowReading();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
   /*
      To enable the program to run when Windows starts, we must create a
      data value under the
      HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Run
      registry key with a name identifying the program, and a string
      giving the full path to the executable file.
      To stop the program running when Windows starts, we must delete the
      data value.
      TRegistry is a VCL component that encapsulates registry functions.
   */

   TRegistry *theReg = new TRegistry();
   theReg->RootKey = HKEY_CURRENT_USER;
   theReg->OpenKey("Software\\Microsoft\\Windows\\CurrentVersion\\Run",false);

   if (theReg->ValueExists("Taize"))
   {
      if (chkWindowsStartup->Checked)
      {
         /*
            The key exists, but does it point to the current instance of the program?
            If not, does the user want to run this instance instead of the instance
            already run by the key? If the user does want to do this, change the key
            to point to this instance of the program.
         */
         if (theReg->ReadString("Taize") != Application->ExeName)
            if (Application->MessageBoxA("A different copy of Daily Reading from Taize is set to run at startup. \nWould you like to set this copy to be run instead?","Warning!",MB_ICONEXCLAMATION | MB_YESNO | MB_DEFBUTTON2) == mrYes)
               theReg->WriteString("Taize",Application->ExeName);
      }
      else
      {
         /*
            If the key exists and points to the current instance of the program,
            remove the key.
         */
         if (theReg->ReadString("Taize") == Application->ExeName)
            theReg->DeleteValue("Taize");
      }
   }
   else
   {
      /*
         If no key exists, but the user has set the option to run@startup,
         create the necessary key.
      */
      if (chkWindowsStartup->Checked)
         theReg->WriteString("Taize",Application->ExeName);
   }

   delete theReg;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuAboutClick(TObject *Sender)
{
   TfrmAbout *frmAbout = new TfrmAbout(this);
   frmAbout->ShowModal();
   delete frmAbout;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ShowReading(void)
{
   // Create a file stream for the 'readings' data file.
   ifstream readings((ExtractFilePath(Application->ExeName) + "readings.dat").c_str());
   if (!readings)
      return;
   char *buffer;
   while (!readings.eof())
   {
      buffer = new char[MAX_CHARS];
      readings.getline(buffer,MAX_CHARS - 1);
      if (AnsiString(buffer) == ReadingToGet)
      {
         ReadingFound = true;
         delete buffer;
         buffer = new char[MAX_CHARS];
         readings.getline(buffer,MAX_CHARS - 1);
         lblReference->Caption = AnsiString(buffer);
         lblReference->Left = 329 - lblReference->Width;
         delete buffer;
         buffer = new char[MAX_CHARS];
         readings.getline(buffer,MAX_CHARS - 1);
         redReading->Lines->Add(PurifyReading(buffer));
         delete buffer;
         return;
      }
      delete buffer;
   }

}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormActivate(TObject *Sender)
{
   if (!ReadingFound)
      if (Application->MessageBoxA("The reading file is out of date. Would you like to download a new reading file now?","Download Reading File?",MB_YESNO) == mrYes)
         DownloadReadings();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::DownloadReadings(void)
{
   // Download Reading file.
   TfrmDownload *frmDownload = new TfrmDownload(this);
   frmDownload->ShowModal();
   delete frmDownload;
   // We should now have a new reading file, so call the display function again.
   ShowReading();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuContentsClick(TObject *Sender)
{
   Application->HelpCommand(HELP_FINDER,0);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuHowtoUseHelpClick(TObject *Sender)
{
   Application->HelpCommand(HELP_HELPONHELP,0);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::mnuSearchforHelpOnClick(TObject *Sender)
{
   Application->HelpCommand(HELP_FINDER,0); 
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ActionCopyReadingExecute(TObject *Sender)
{
   /*
      By using a TActionList, we can replace the default copy command
      for the Memo component with our own custom version.
      The custom version also includes the date, and reference for the reading
      in the copied text.
   */

   int x, readingLen;

   // Get an array of chars containing the reading...
   char *tempReading = redReading->Lines->GetText();

   // If there is no reading, then we can't copy it! :)
   if (!tempReading)
      return;

   // Work out the number of chars in the reading...
   readingLen = 0;
   while (tempReading[readingLen] != '\0')
      readingLen++;

   // Work out the number of chars we need for the text...
   x = lblDate->Caption.Length() + lblReference->Caption.Length()
       + 7 + readingLen;

   // Allocate the number of chars to our pointer...
   char *temp = new char[x];

   // Build the text we will export...
   temp = strcpy(temp,lblDate->Caption.c_str());
   temp = strcat(temp,"   ");
   temp = strcat(temp,lblReference->Caption.c_str());
   temp = strcat(temp,"\r\n");
   temp = strcat(temp,tempReading);
   temp = strcat(temp, "\0");

   // Send the array of chars to the Windows clipboard...
   Clipboard()->SetTextBuf(temp);

   // Cleanup...
   delete temp, tempReading;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::WhenIdle(TObject * Sender, bool & done)
{
   pmnuCopy->Enabled = ReadingFound;
   mnuCopy->Enabled = ReadingFound;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ActionShowFileDateExecute(TObject *Sender)
{
   /*
      This Action, displays the date of the first reading contained in
      the reading file. May be useful for checking if the reading file
      is out of date.
   */
   ifstream readings((ExtractFilePath(Application->ExeName) + "readings.dat").c_str());
   if (!readings)
      return;
   char *buffer, tempMonth[3], tempDay[3];
   buffer = new char[MAX_CHARS];
   readings.getline(buffer,MAX_CHARS - 1);
   tempDay[0] = buffer[0];
   tempDay[1] = buffer[1];
   tempDay[2] = '\0';
   tempMonth[0] = buffer[3];
   tempMonth[1] = buffer[4];
   tempMonth[2] = '\0';
   unsigned short theYear, theMonth, theDay;
   Date().DecodeDate(&theYear,&theMonth,&theDay);
   TDateTime fileStartDate(theYear,(unsigned short)atoi(tempMonth),(unsigned short)atoi(tempDay));

   Application->MessageBoxA((fileStartDate.FormatString("ddd d mmm yyyy")).c_str() ,"Reading file start date",MB_OK);
   delete buffer;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::ActionDeleteReadingsExecute(TObject *Sender)
{
   /*
      This Action deletes the reading file. May be usefull if the file
      is corrupted or out of date.
   */
   if (Application->MessageBoxA("Are you sure you want to delete the current reading file?","Delete reading file",MB_ICONEXCLAMATION | MB_YESNO | MB_DEFBUTTON2) == mrYes)
      DeleteFile(ExtractFilePath(Application->ExeName) + "readings.dat");
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::redReadingKeyPress(TObject *Sender, char &Key)
{
   /*
      If the user has pressed the space key or return key while focus
      was on redReading, behave as if the user had clicked cmdClose.
   */
   if (Key == VK_SPACE || Key == VK_RETURN)
   {
      Key = NULL;
      cmdClose->Click();
   }
}
//---------------------------------------------------------------------------
String __fastcall TfrmMain::PurifyReading(char *oldReading)
{
   /*
      This function removes all control characters from the reading before
      it is displayed. Depending on how carefully I have created the reading
      file from the word document from Taize, a few control characters may slip
      through.
   */

   // Create a buffer for the purified reading.
   char *buffer;
   buffer = new char[MAX_CHARS + 1];

   int CurrentPosition = 0, skipped = 0;

   /*
      This loop steps through each character in the reading, and checks it to
      make sure that it is either alpha-numeric, white-space, or punctuation.
      If the characters is not one of these three types, we ignore it and carry
      on. The checking is performed by three standard library functions:
      isalnum(), ispunct(), and isspace().

      We also keep track of how many characters we ignore, as this has an effect
      on the position of the current character from the old reading in the
      purified reading.
   */
   
   while (oldReading[CurrentPosition] != NULL)
   {
      if (isalnum(oldReading[CurrentPosition]) || ispunct(oldReading[CurrentPosition]) || isspace(oldReading[CurrentPosition]))
         buffer[CurrentPosition - skipped] = oldReading[CurrentPosition]; // Add character to purified reading buffer.
      else
         skipped++; // Skip this character and increment the count of skipped characters.

      CurrentPosition++; // Move on to the next character.
   }

   buffer[CurrentPosition - skipped] = '\0'; // Nul terminate the buffer.

   String NewReading(buffer); // Convert the purified reading buffer to an AnsiString.
   delete buffer;
   return NewReading; // Return the purified reading.

}
//---------------------------------------------------------------------------

